// SPDX-License-Identifier: LGPL-3.0-or-later
// See Notices.txt for copyright information
use crate::prelude::*;

#[hdl]
pub struct ReadyValid<T> {
    pub data: HdlOption<T>,
    #[hdl(flip)]
    pub ready: Bool,
}

impl<T: Type> ReadyValid<T> {
    #[hdl]
    pub fn fire(expr: Expr<Self>) -> Expr<Bool> {
        #[hdl]
        let fire: Bool = wire();
        #[hdl]
        match expr.data {
            HdlNone => connect(fire, false),
            HdlSome(_) => connect(fire, expr.ready),
        }
        fire
    }
    #[hdl]
    pub fn map<R: Type>(
        expr: Expr<Self>,
        f: impl FnOnce(Expr<T>) -> Expr<R>,
    ) -> Expr<ReadyValid<R>> {
        let data = HdlOption::map(expr.data, f);
        #[hdl]
        let mapped = wire(ReadyValid[Expr::ty(data).HdlSome]);
        connect(mapped.data, data);
        connect(expr.ready, mapped.ready);
        mapped
    }
}
